#include<avr/io.h>
#include"MMC_SD/MMC_SD.h"
#include"FAT/FAT.h"
#include <util/delay.h>

// all on PORTC
#define	LCD_CD	PC7   
#define	LCD_WR  PC6
#define	LCD_RD  PC5
#define	LCD_CS	PC4
#define	LCD_RES	PC2

#define CLR_CD PORTC &= ~(1<<LCD_CD);
#define SET_CD PORTC |=  (1<<LCD_CD);

#define CLR_CS PORTC &= ~(1<<LCD_CS);
#define SET_CS PORTC |=  (1<<LCD_CS);

#define CLR_RESET PORTC &= ~(1<<LCD_RES);
#define SET_RESET PORTC |=  (1<<LCD_RES);

// pin functions
#define CLR_WR PORTC &= ~(1 << LCD_WR);
#define SET_WR PORTC |=  (1 << LCD_WR);
#define CLR_RD PORTC &= ~(1 << LCD_RD);
#define SET_RD PORTC |=  (1 << LCD_RD);

#define	BLACK	  0x0000
#define	BLUE	  0x001F
#define	RED 	  0xF800
#define	GREEN   0x07E0
#define CYAN	  0x07FF
#define MAGENTA 0xF81F
#define YELLOW  0xFFE0
#define WHITE	  0xFFFF

#define uint8 unsigned char
#define uint16 unsigned int
#define uint32 unsigned long

unsigned char * PATH = "\\batch";

extern uint16 SectorsPerClust;
extern uint16 FirstDataSector; 
extern uint8 FAT32_Enable;

struct FileInfoStruct FileInfo;

struct direntry PictureInfo;
uint16 totalfiles = 0;

void Delay(uint16 n)
{
	while (n--)
	asm("nop");
}

/*************************************************/
void delay(unsigned int t)
{
	unsigned char t1;
	while(t--)
	for(t1=11;t1>0;t1--)
	{;}      //0.1ms
}
/*************************************************/
void write_command(unsigned int command)
{
	CLR_CS;
	CLR_CD;
	SET_RD;
	SET_WR;

	PORTA=command>>8;

	CLR_WR;
	SET_WR;

	PORTA=command;

	CLR_WR;
	SET_WR;
	SET_CS;
}

/*************************************************/
void write_data(unsigned int data)
{
	CLR_CS;
	SET_CD;
	SET_RD;
	SET_WR;

	PORTA=data>>8;

	CLR_WR;
	SET_WR;

	PORTA=data;

	CLR_WR;
	SET_WR;
	SET_CS;
}
/*************************************************/
void initialization()
{
	CLR_RD;
	CLR_WR;
	SET_CS;
	SET_CD;
	PORTA=0x00;

	CLR_RESET;
	delay(10000);
	delay(10000);
	SET_RESET;
	delay(500);

	//--------------------Display control setting--------------------------------
	write_command( 0x0001); //output direct
	write_data( 0x0100);
	write_command( 0x0002); //line inversion
	write_data( 0x0700);
	write_command( 0x0003); //entry mode (65K,write ram direction ,BGR)
	write_data( 0x1030);
	write_command( 0x0008); //Front porch=3,Back porch=2
	write_data( 0x0302);
	write_command( 0x0009); //scan cycle
	write_data( 0x0000);
	write_command( 0x000A); //Fmark On
	write_data( 0x0000);
	//--------------------initial power status ----------------------------------
	write_command( 0x0010); //power control1
	write_data( 0x0000);
	write_command( 0x0011); //power control2
	write_data( 0x0007);
	write_command( 0x0012); //power control3
	write_data( 0x0000);
	write_command( 0x0013); //power control4
	write_data( 0x0000);
	delay(1000);			 //100ms	
	//--------------------Power supply startup 1 setting-------------------------
	write_command( 0x0010); //power control1
	write_data( 0x14b0);
	delay(500); 			 //50ms
	write_command( 0x0011); //power control2
	write_data( 0x0007);
	delay(500);				 //50ms
	write_command( 0x0012); //power control3
	write_data( 0x008e);
	write_command( 0x0013); //power control4
	write_data( 0x0C00);
	write_command( 0x0029); //Vcomh setting
	write_data( 0x0015);
	delay(500);				 //50ms
	//-----------------default gamma (CABC OFF)-----------------------------------
	write_command(0x0030);
	write_data(0x0000);
	write_command(0x0031);
	write_data(0x0107);
	write_command(0x0032);
	write_data(0x0000);
	write_command(0x0035);
	write_data(0x0203);
	write_command(0x0036);
	write_data(0x0402);
	write_command(0x0037);
	write_data(0x0000);
	write_command(0x0038);
	write_data(0x0207);
	write_command(0x0039);
	write_data(0x0000);
	write_command(0x003c);
	write_data(0x0203);
	write_command(0x003d);
	write_data(0x0403);
	//-----------------default gamma (CABC OFF)-----------------------------------
	//--------------------Display window 240 X 320-------------------------------
	write_command( 0x0050);
	write_data( 0x0000);
	write_command( 0x0051);
	write_data( 0x00ef);
	write_command( 0x0052);
	write_data( 0x0000);
	write_command( 0x0053);
	write_data( 0x013f);
	//----------------------------------------------------------------------------
	write_command( 0x0060); //Gate scan control
	write_data( 0xa700);  
	write_command( 0x0061); // Non-display Area setting
	write_data( 0x0001);
	write_command( 0x0090); //RTNI setting
	write_data( 0x0010);
	write_command( 0x0007); //display on
	write_data( 0x0133);
	delay(500);
	write_command(0x22);
}
/*************************************************/
void Display_Home()
{
	write_command( 0x0020);//Horizontal Address
	write_data( 0x0000);
	write_command( 0x0021);//Vertical Address
	write_data( 0x0000);
}
/*************************************************/
void display_rgb(unsigned int data)
{
	unsigned int i,j;
	Display_Home();

	for(i=0;i<320;i++)
	{
		for(j=0;j<240;j++)
		{
			write_data(data);
		}
	}
	write_command(0x22);
}
/*************************************************/
void Picture_Slide_Show()
{
	uint8 i;              //loop variable
	uint16 j;             //loop variable
	uint32 p;             //cluster
	uint16 *buffer;				//buffer
	uint16 pics = 1;
	uint16 side_show_flag = 1;
	uint8 sector;
	uint32 pixels;

	if (totalfiles == 0)
	return;

	buffer = malloc(512);
	if (buffer == 0)
	return;

next: 	//label for "goto"

	Search(PATH, &PictureInfo, &pics);                                         //find the file
	p = PictureInfo.deStartCluster + (((uint32)PictureInfo.deHighClust) << 16);//the first cluster of the file

	sector = 0;

	FAT_LoadPartCluster(p, sector, buffer);	//read a sector


	pixels = (uint32) 320 * 240; // total # of pixels to write

	Display_Home();
	write_command(0x22);
	
	
	j = 0; // byte count
	
	while (1)
	{
		write_data(buffer[j]);  // write 16 bits
		pixels--;               // which is one pixel
		j++;                    // word count

		if (pixels == 0)        
		break;

		if (j == 256)           // time for a new sector     
		{
			sector++;
			if (sector == SectorsPerClust)
			{
				p = FAT_NextCluster(p);				// read next cluster
				sector = 0;
			}

			FAT_LoadPartCluster(p, sector, buffer);	// read a sector
			j = 0;
		}
	}

	if (side_show_flag)
	{
		for (i=0;i<100;i++)	// delay for a while
		Delay(0xffff);

		pics++;					// inc pic number
		if (pics > totalfiles)	// if last
		pics = 1;			// wrap around

		goto next;				// go show next pic
	}
	free(buffer);
}
/*************************************************/
void LCD_test()
{
	unsigned int i,j;
	Display_Home();
	write_command(0x22);

	for(i=0;i<320;i++)
	{
		for(j=0;j<240;j++)
		{
			if(i>279)write_data(BLACK);
			else if(i>239)write_data(BLUE);
			else if(i>199)write_data(GREEN);
			else if(i>159)write_data(CYAN);
			else if(i>119)write_data(RED);
			else if(i>79)write_data(MAGENTA);
			else if(i>39)write_data(YELLOW);
			else write_data(WHITE);
		}
	}
}
/*************************************************/
int main()
{

	PORTD = 0xF8;	    // pull-up on switches, all others off
	DDRD  = 0x0F;   	// VPP and led pins output

	DDRA = 0xFF;    	// low data port, all outputs
	DDRB = 0xFF;    	// all outputs
	
	PORTC = 0xFE;	    // all pins high
	DDRC  = 0xFE;	    // all outputs

  PORTB = 0x20;     // turn on backlight

	uint8 retry = 0;

	OSCCAL = 0x00;    //in order to operate some low speed card the initialization should run at lowest speed

	Delay(0xffff);

	MMC_SD_Init();    //SPI initialize

	Delay(0xffff);

	// Initialize the display
	initialization();

	LCD_test();

	sd_restart: 	//label for "sd_restart"

	while (MMC_SD_Reset())  // Initialize SD Card
	{
		retry++;
		if (retry > 254)
		{
			while (1)
			{
				goto sd_restart;				// try again
			}
		}
	}

	//	OSCCAL = 0xff;      //run at high speed for normal operation for SD Card
	OSCCAL = 0x00;      //run at high speed for normal operation for SD Card


	Delay(0xffff);      

	if (FAT_Init())   	//initialize file system - FAT16 and FAT32 are supported
	{
		while (1)
		{
			goto sd_restart;				// try again
		}
	}

	Search(PATH, &PictureInfo, &totalfiles);

	Picture_Slide_Show();

	while (1)
	{
	}
	return 0;

}

